/* ----------------------------------------------------------------------
   The SINUS Firewall -- a TCP/IP packet filter for Linux

   Written within the SINUS project at the University of Zurich,
   SWITCH, Telekurs Payserv AG, ETH Zurich.
   originally based on the sf Firewall Software (C) 1996 by Robert
   Muchsel and Roland Schmid.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   SINUS Firewall resources:
   SINUS Homepage: http://www.ifi.unizh.ch/ikm/SINUS/
   Firewall Homepage: http://www.ifi.unizh.ch/ikm/SINUS/firewall.html
   Frequently asked questions: http://www.ifi.unizh.ch/ikm/SINUS/sf_faq.html
   Mailing list for comments, questions, bug reports: firewall@ifi.unizh.ch
   ----------------------------------------------------------------------  */


/*
 * $Id: sf_filter.h,v 1.20 1995/08/05 11:37:46 robby Rel $
 */

#ifndef _SF_FILTER_H
#define _SF_FILTER_H

#include <linux/firewall.h>
#include <linux/timer.h>
#include "sf_global.h"

/*
 *    Variables for interaction with the driver
 */

/* filter use count */
extern int sf_fw_enabled;

/* packet filtering routine */
extern int sf_check_packet(struct iphdr *, struct device *, int,
			   struct sk_buff *);

/* configuration update */
extern int sf_write_config(const char *, int);

/* temporary storage for packets to be sent to the daemon */
extern struct sf_proc sf_log_data;

/* wait queue for synchronization between kernel filter module and daemon */
extern struct wait_queue *sf_log_entry;

/* delete all timers, must be called before unloading the module */
extern void sf_del_all_timers(void);

/* initialize all data structures */
extern int sf_init(void);

/* read actual configuration */
extern struct sf_fw *prio_rules;
extern struct sf_fw *rules;
extern struct sf_tcp_connection *conns;
extern struct sf_fw *rule_first_next;
extern unsigned long conn_id;

extern inline long sf_rule_first(void) 
{
  /* return pointer to first rule */

  rule_first_next = prio_rules;
  return (long) rule_first_next;
}

extern inline long sf_rule_next(void)
{
/* return pointer to next rule */
  if (rule_first_next != NULL)
    rule_first_next = rule_first_next->fw_next;

  return (long) rule_first_next;
}

extern inline long sf_conn_first(void)
{
  /* return pointer to first active TCP connection */
  if (conns != NULL)
    conn_id = conns->id;
  else
    conn_id = 0;
#ifdef SF_DEBUG_LLIST
  printk("sf_conn_first: @  = %08x\n", (unsigned int) conns);
  if (conn_first_next != NULL) {
    printk("sf_conn_first: id = %08x\n", (unsigned int) conns->id);
    printk("sf_conn_first: <- = %08x\n", (unsigned int) conns->before);
    printk("sf_conn_first: -> = %08x\n", (unsigned int) conns->after);
  }
#endif

  return (long) conns;
}

extern inline long sf_conn_next(void)
{
  /* get pointer to next active TCP connection */
  /* we don't save a pointer to the list, because the list could change */
  /* between calls (i.e. an entry expires) */
  struct sf_tcp_connection *sftcp;

  sftcp = conns;
  while (sftcp != NULL && sftcp->id <= conn_id)
    sftcp = sftcp->after;
  if (sftcp != NULL)
    conn_id = sftcp->id;
  else
    conn_id = 0;

#ifdef SF_DEBUG_LLIST
  printk("sf_conn_next: @  = %08x\n", (unsigned int) sftcp);
  if (sftcp != NULL) {
    printk("sf_conn_next: id = %08x\n", (unsigned int) sftcp->id);
    printk("sf_conn_next: <- = %08x\n", (unsigned int) sftcp->before);
    printk("sf_conn_next: -> = %08x\n", (unsigned int) sftcp->after);
  }
#endif

  return (long) sftcp;
}

extern struct firewall_ops sf_fw_ops;
extern struct firewall_ops sf_blocking_fw_ops;

/* checks wether addr matches sf_addr entries between idx and idx+cnt-1 */
int sf_addr_match(long idx, long cnt, __u32 addr, int port, int negative);

#endif				/* _SF_FILTER_H */
